@@ -3,7 +3,7 @@ package Net::Curl::Promiser;
 use strict;
 use warnings;
 
-our $VERSION = '0.07_04';
+our $VERSION = '0.07_05';
 
 =encoding utf-8
 
@@ -366,6 +366,9 @@ sub _socket_fn {
     }
     elsif ($action == Net::Curl::Multi::CURL_POLL_REMOVE) {
         $self->_STOP_POLL($fd);
+
+        # In case we got a read and a remove right away.
+        $self->_process_pending();
     }
     else {
         warn "$self: Unrecognized action $action on FD $fd\n";
@@ -377,21 +380,37 @@ sub _socket_fn {
 sub _finish_handle {
     my ($self, $easy, $cb_idx, $payload) = @_;
 
-    delete $self->{'to_fail'}{$easy};
+    # As of v0.43 Net::Curl doesn’t report errors in callbacks.
+    my $ok = eval {
+        print STDERR "=== Finished: $easy\n";
 
-    $self->{'multi'}->remove_handle( $easy );
+            delete $self->{'to_fail'}{$easy};
 
-    if ( my $cb_ar = delete $self->{'callbacks'}{$easy} ) {
-        $cb_ar->[$cb_idx]->($payload);
-    }
-    elsif ( my $deferred = delete $self->{'deferred'}{$easy} ) {
-        if ($cb_idx) {
-            $deferred->reject($payload);
-        }
-        else {
-            $deferred->resolve($payload);
-        }
-    }
+            $self->{'multi'}->remove_handle( $easy );
+
+            if ( my $cb_ar = delete $self->{'callbacks'}{$easy} ) {
+        print STDERR "=== Callback $easy: $cb_idx\n";
+                $cb_ar->[$cb_idx]->($payload);
+        print STDERR "=== after callback $easy: $cb_idx\n";
+            }
+            elsif ( my $deferred = delete $self->{'deferred'}{$easy} ) {
+                if ($cb_idx) {
+                    $deferred->reject($payload);
+                }
+                else {
+                    $deferred->resolve($payload);
+                }
+            }
+            else {
+
+                # This shouldn’t happen, but just in case:
+                require Data::Dumper;
+                print STDERR Data::Dumper::Dumper( ORPHAN => $easy => $payload );
+            }
+        print STDERR "=== Done Finished: $easy\n";
+    };
+
+    warn if !$ok;
 
     return;
 }
@@ -414,6 +433,7 @@ sub _process_pending {
 
     $self->_clear_failed();
 
+print STDERR "==== start checking for finished easies\n";
     while ( my ( $msg, $easy, $result ) = $self->{'multi'}->info_read() ) {
 
         if ($msg != Net::Curl::Multi::CURLMSG_DONE()) {
@@ -425,6 +445,7 @@ sub _process_pending {
             ($result == 0) ? ( 0 => $easy ) : ( 1 => $result ),
         );
     }
+print STDERR "==== done checking for finished easies\n";
 
     return;
 }
